function h = histogram_custom(BinEdgesString, BinEdges, BinCountsString, BinCounts, varargin)
% This function was written as a Octave compatibility replacement for the
% Matlab's "histogram" function. It is not a full-fledged replacement, but
% only for the special use cases of IFM Analyzer application.


%  Check the whether the input arguments were given in a name-value pair format 
if ~ischar(BinCountsString) || ~ischar(BinEdgesString)
   error('The input arguments must be given in a name-value pair format.'); 
end

% check the validity of the strings of the name-value pairs
if ~strcmp('BinCounts', BinCountsString) || ~strcmp('BinEdges', BinEdgesString)

   error('The input arguments must be given in the following way: "BinEdges", BinEdges, ''BinCounts'', BinCounts'); 
end

if isempty(varargin)
    try
        % use the Matab's built-in "histogram" function
        h=histogram(BinEdgesString,BinEdges,BinCountsString,BinCounts);
        %error('Some string.'); % only for testing purposes
    catch
        % use the Octave compatible "bar" function in a similar manner to the
        % "histogram" function
        h=bar(BinEdges(1:end-1), BinCounts, 'histc');
        set(h, 'FaceColor', [ 0    0.4470    0.7410]);
        % set(h, 'FaceAlpha', 0.6);   % does not work in Octave
    end
else
    graph=varargin{1}; % pass the axis handle
    try
        error('Some text.') % for testing only
        
        % original line in the "lineDiameter_fit_inputDialog" function
        % histogram(graph, ThompsonPrec);
        % after modification:
        histogram(graph, 'BinEdges', BinEdges, 'BinCounts', BinCounts);
    catch
        % use the Octave compatible "bar" function in a similar manner to the
        % "histogram" function
        h=bar(graph, BinEdges(1:end-1), BinCounts, 'histc');
        set(h, 'FaceColor', [ 0    0.4470    0.7410]);
        % set(h, 'FaceAlpha', 0.6);   % does not work in Octave
    end
end

end
