function modelFunc_Y_convolved_binned = binModelFunction(modelFunc_Y_convolved, histBinN, binRefinement)
% This function numerically integrates the model function over the
% histogram bins. The histogram bins must be equally sized.
% the integration is a simple average of the model function sampling points
% belonging to the different bins.

% indices of the model function values belonging to the first bin
intIndices_firstBin=1:binRefinement;

% for each bin, these values needs be added to the aforementioned indices
% to get the indices of the model function values belonging to the bins
intIndices_binIncrement=((1:histBinN)'-1)*binRefinement;

% indices of the model function values belonging to the bins
linIndices_forPixelization=repmat(intIndices_firstBin,histBinN,1)+repmat(intIndices_binIncrement,1,binRefinement);

% constants for the numerical integration
numIntConstant=ones(size(linIndices_forPixelization))/binRefinement;

% model function values integrated over the bins
modelFunc_Y_convolved_binned=sum(numIntConstant.*modelFunc_Y_convolved(linIndices_forPixelization),2);


end

