function [ modelFunc_Y ] = calculateModelFunction( sampleType, modelFuncType, modelFunctionSettings, samplingSettings)
% This function calculates the model function to be fit on the histogram data
% of the measurement. This model function shall be convolved and binned
% before the comparing it with the histogram of the original data.


%% Model function prior to the convolution
% calculating the values of the selected model function prior to the convolution
switch sampleType
    case 'single line'
        %  single line structures with different model functions
        switch modelFuncType
            case 'Circle'
                modelFunc_Y=modelFunctions.circle(modelFunctionSettings, samplingSettings);
            case 'Disk'
                modelFunc_Y=modelFunctions.disk(modelFunctionSettings, samplingSettings);
            case 'Ring'
                modelFunc_Y=modelFunctions.ring(modelFunctionSettings, samplingSettings);
            case 'Gaussian'
                modelFunc_Y=modelFunctions.Gaussian(modelFunctionSettings, samplingSettings);
            case 'Lorentzian'
                modelFunc_Y=modelFunctions.Lorentzian(modelFunctionSettings, samplingSettings);
            case 'Rectangular'
                modelFunc_Y=modelFunctions.rectangle(modelFunctionSettings, samplingSettings);
            case 'Rectangular gap'
                modelFunc_Y=modelFunctions.rectangularGap(modelFunctionSettings, samplingSettings);
            otherwise
                error('Invalid model function was given.')
        end
    case 'double lines'
        %  double line structure
        
        % the first peak:
        modelFunctionSettings_1=modelFunctionSettings;
        modelFunctionSettings_1.position=modelFunctionSettings.position1;
        modelFunc_Y_1=calculateModelFunction( 'single line', modelFuncType, modelFunctionSettings_1, samplingSettings);
        
        % the second peak:
        modelFunctionSettings_2=modelFunctionSettings;
        modelFunctionSettings_2.position=modelFunctionSettings.position2;
        modelFunc_Y_2=calculateModelFunction( 'single line', modelFuncType, modelFunctionSettings_2, samplingSettings);
        
        % adjust the ratio of the peaks:
        if isfield(modelFunctionSettings, 'height') || isfield(modelFunctionSettings, 'area')
            modelFunc_Y_1=modelFunc_Y_1/2;
            modelFunc_Y_2=modelFunc_Y_2/2;
        elseif (isfield(modelFunctionSettings, 'height1') && isfield(modelFunctionSettings, 'height2'))
            modelFunc_Y_1=modelFunc_Y_1*modelFunctionSettings.height1/(modelFunctionSettings.height1+modelFunctionSettings.height2);
            modelFunc_Y_2=modelFunc_Y_2*modelFunctionSettings.height2/(modelFunctionSettings.height1+modelFunctionSettings.height2);
        else
            error('Please set a parameter with the double line peaks can be adjusted.')
        end
        
        % the resultant double line structure:
        modelFunc_Y=modelFunc_Y_1+modelFunc_Y_2;
        
    case 'gap'
        %  rimmed gap structure
        
        % TODO: should be considered moving the gap's logic here from
        % 'modelFunctions' to be consistent with that of the double line.
        switch modelFuncType
            case 'Gap'  % Rectangular Gap
                modelFunc_Y=modelFunctions.rectangularGap(modelFunctionSettings, samplingSettings);
            otherwise
                error('Invalid model function was given.')
        end
    otherwise
        error('Unknown sample type');
end

