function [lineDiam_lastParams, initLineFigure, skipPolinomialFittingBool] = manualLinearSetting(LOC_X, LOC_Y, initLine_linearFit, initParams, automaticProcessBool)
% This function is for manually adjusting the initially fit line and the
% for setting the region to be cut out in a pop-up window (if not disabled
% with the automatic data processing). The parameters of the cut-out region
% is also asked in a pop up window. The figure is refreshed after every
% parameter modification.

% angle conversion for the user input and output
    function outputAngle=angleInConversion(InputAngle)
        outputAngle=(InputAngle)*pi/180;
    end

    function outputAngle=angleOutConversion(InputAngle)
        outputAngle=InputAngle*180/pi;
    end

%% Initialization of some parameters
xrange=initParams.xrange;
yrange=initParams.yrange;
sig=initParams.sig;
sampLineDistThreshold=initParams.sampLineDistThreshold;
nBins=initParams.nBins;
polyDegree=initParams.polyDegree;
surfFittingPower=initParams.surfFittingPower;

alpha_mean=atan(initLine_linearFit.m)-90*pi/180;              % angle of the fitted line
% because the properties of the atan function
if alpha_mean<-pi/2
    alpha_mean=alpha_mean+180*pi/180;
elseif alpha_mean>pi/2
    alpha_mean=alpha_mean-180*pi/180;
end

dlgBool=true;                               % open later a question dialog
drawArrow = @(x,y,varargin) quiver( x(1),y(1),x(2)-x(1),y(2)-y(1),0, varargin{:} ); % drawing later the arrow showing the central displacement

% the remnant parameters, the center position of the rectangular region
xCenter=mean(LOC_X);    % "x" components of the data points' mean position
yCenter=mean(LOC_Y);    % "y" components of the data points' mean position

%calculating the displacement of the fitted line (the vector pointing from [xCenter,yCenter] to the fitted line and is perpedicular to it)
% parameters of the fitted line, f(x)=m*x+h
m=initLine_linearFit.m;
h=initLine_linearFit.h;

% parameters of the line perpendicular to the fitted one, f(x)=m_perp*x+h_perp
m_perp=tan(atan(m)+pi/2);
h_perp=yCenter-m_perp*xCenter;

% the vector pointing from [xCenter,yCenter] to the fitted line and is perpedicular to it
xDispacement=(h_perp-h)/(m-m_perp)-xCenter;
yDispacement=(h/m-h_perp/m_perp)/(1/m-1/m_perp)-yCenter;
displacement=[xDispacement,yDispacement];  % magnitude of the autocorrelation's displacement

%% Setting the region to be cut out
% drawing the figure opening the dialog until the set parameters are not
% accepted
if automaticProcessBool
    %figure(10)
    initLineFigure=figure('visible','off');
else
    initLineFigure=figure();
end
set(initLineFigure, 'Name', 'Choosing the rectangular region of interest')
while dlgBool
    % plotting the data points and the fitted line
    ax=axes;
    scatter(LOC_X, LOC_Y, 5, 'r', '.')  % plotting the data points
    
    if exist('OCTAVE_VERSION', 'builtin') ~= 0
      % workaround for an Octave bug
      set(ax, 'XLim', [0, 1])
      set(ax, 'YLim', [0, 1])
      set(ax, 'XLimMode', 'Auto')
      set(ax, 'YLimMode', 'Auto')
    end
    hold on;
    % getting the axis limits after the scatter plot:
    xlimit=get(ax, 'XLim');
    ylimit=get(ax, 'YLim');

    % setting the axis limits back to span only the scatter plot:
    set(gca, 'XLim', xlimit, 'YLim', ylimit);
    
    % other axis formatting:
    view(90, 90)                    % exchanging the 'x' and 'y' axes
    daspect([1 1 1])                        % fixing the aspect ration, the axis('equal') and axis('image') does not work correctly
    
    % the displacement vectorstarting and ending coordinates:
    x1=[xCenter, xCenter+displacement(1)];    % starting and ending "x" coordinates of the arrow
    y1=[yCenter, yCenter+displacement(2)];    % starting and ending "y" coordinates of the arrow
    
    % calculate the coordiantes of the rectangle to be plotted:
    x=zeros(5,1);               % "x" coordinates of the vertices of the recttangle bounding the displacement region
    x(1)=x1(2)+(cos(alpha_mean+90*pi/180)*(-xrange)-sin(alpha_mean+90*pi/180)*(+yrange));
    x(2)=x1(2)+(cos(alpha_mean+90*pi/180)*(+xrange)-sin(alpha_mean+90*pi/180)*(+yrange));
    x(3)=x1(2)+(cos(alpha_mean+90*pi/180)*(+xrange)-sin(alpha_mean+90*pi/180)*(-yrange));
    x(4)=x1(2)+(cos(alpha_mean+90*pi/180)*(-xrange)-sin(alpha_mean+90*pi/180)*(-yrange));
    x(5)=x1(2)+(cos(alpha_mean+90*pi/180)*(-xrange)-sin(alpha_mean+90*pi/180)*(+yrange));
    y=zeros(5,1);               % "y" coordinates of the vertices of the rectangle bounding the displacement region
    y(1)=y1(2)+(sin(alpha_mean+90*pi/180)*(-xrange)+cos(alpha_mean+90*pi/180)*(+yrange));
    y(2)=y1(2)+(sin(alpha_mean+90*pi/180)*(+xrange)+cos(alpha_mean+90*pi/180)*(+yrange));
    y(3)=y1(2)+(sin(alpha_mean+90*pi/180)*(+xrange)+cos(alpha_mean+90*pi/180)*(-yrange));
    y(4)=y1(2)+(sin(alpha_mean+90*pi/180)*(-xrange)+cos(alpha_mean+90*pi/180)*(-yrange));
    y(5)=y1(2)+(sin(alpha_mean+90*pi/180)*(-xrange)+cos(alpha_mean+90*pi/180)*(+yrange));
    
    % the fitted linear line:
    linFunc_x=linspace(xlimit(1), xlimit(2), 2);
    linFunc_y=initLine_linearFit.m*linFunc_x+initLine_linearFit.h;
    
    plot(linFunc_x, linFunc_y, 'color', 'c')             % plotting the fitted linear
    legend('off');
    hold on;
    drawArrow(x1,y1, 'linewidth',3, 'MaxHeadSize', 1, 'color','g');     % drawing arrow showing the central displacement
    hold on;
    plot(x, y, 'color', 'c');   % plotting the rectangle bounding the displacement region
    hold on
    viscircles([xCenter, yCenter], sig, 'Color', 'm', 'Linewidth', 1);  % plotting the circle showing the standard deviation of the Gaussian blur
    hold off;
    %disp(['Angle of fitted curve: ',num2str(angleOutConversion(alpha_mean)), '???'])
    
    % booolean, if true it accepts the initial straight line as the central
    % line and will not perform fitting on the localizations
    skipPolinomialFittingBool=false;
    
    % question dialog in which one can accept the current angle,
    % set a new one manually or quit from the program
    if automaticProcessBool
        choice='Accept parameters';
    else
        % older version with fewer buttons: questdlg
        [choice, skipPolinomialFittingBool] = choosedialog({['Current angle of the initial line: ', num2str(angleOutConversion(alpha_mean)), '???'];...
            ['Current [x, y] displacement the rectangular region is: ', num2str(displacement), ' nm'];...
            ['Current length of the rotated rectangular region is: ', num2str(xrange), ' nm'];...
            ['Current height of the rotated rectangular region is: ', num2str(yrange), ' nm'];...
            ['Current standard deviation of the Gaussian blur is: ', num2str(sig), ' nm'];...
            ['Current maximum line-localization distance is: ', num2str(sampLineDistThreshold), ' nm'];...
            ['Current number of bins on the histogram is: ', num2str(nBins)];...
            ['Current polynomial degree is: ', num2str(polyDegree)];...
            ['Current power for curve fitting on surface is: ' num2str(surfFittingPower)]}, ...
            'Parameters of the curve fitting region', 'Quit', 'Accept parameters', 'Set parameters manually', 'Draw line');
    end

    switch choice
        case 'Accept parameters'
            dlgBool=false;
        case 'Set parameters manually'
            prompt = {'Angle of the initial line: ', '[x, y] displacement the rectangular region [nm]:', 'Length of the rotated rectangular region [nm]:', 'Height of the rotated rectangular region [nm]:', 'Standard deviation of the Gaussian blur [nm]:', 'Maximum line-localization distance [nm]:', 'Number of bins on the histogram:', 'Degree of the polynomial to be fit:', 'Power minimizing the fitted curve along the surface:'}; % name of the input fields
            dlgTitle = 'Set a rectangular region';   % title of the dialog
            nLines = 1;     % one line for each input fields
            % the default values on the dialog
            defaultAns = {num2str(angleOutConversion(alpha_mean)), num2str(displacement), num2str(xrange), num2str(yrange), num2str(sig), num2str(sampLineDistThreshold), num2str(nBins), num2str(polyDegree), num2str(surfFittingPower)};
            % getting the new parameters
            answer = inputdlg(prompt,dlgTitle,nLines,defaultAns);  % the set "z" boundary values in a cell of strings
            alpha_mean=angleInConversion(str2num(char(answer{1})));
            displacement=str2num(answer{2});
            xrange=str2num(answer{3});
            yrange=str2num(answer{4});
            sig=str2num(answer{5});
            sampLineDistThreshold=str2num(answer{6});
            nBins=str2num(answer{7});
            polyDegree=str2num(answer{8});
            surfFittingPower=str2num(answer{9});
            % the new parameters of the line
            initLine_linearFit.m=tan(alpha_mean+90*pi/180);
            % should go through the displaced center
            initLine_linearFit.h=yCenter-xCenter*tan(alpha_mean+90*pi/180)-displacement(1)*tan(alpha_mean+90*pi/180)+displacement(2);
        case 'Draw line'
            [drawnLine.m, drawnLine.h] = drawStraightLine(initLineFigure);
            
            alpha_mean=atan(drawnLine.m)-90*pi/180;              % angle of the fitted line
            % because the properties of the atan function
            if alpha_mean<-pi/2
                alpha_mean=alpha_mean+180*pi/180;
            elseif alpha_mean>pi/2
                alpha_mean=alpha_mean-180*pi/180;
            end
            
            % parameters of the line perpendicular to the fitted one, f(x)=m_perp*x+h_perp
            m_perp=tan(atan(drawnLine.m)+pi/2);
            h_perp=yCenter-m_perp*xCenter;
            
            % the vector pointing from [xCenter,yCenter] to the fitted line and is perpedicular to it
            xDispacement=(h_perp-drawnLine.h)/(drawnLine.m-m_perp)-xCenter;
            yDispacement=(drawnLine.h/drawnLine.m-h_perp/m_perp)/(1/drawnLine.m-1/m_perp)-yCenter;
            displacement=[xDispacement,yDispacement];  % magnitude of the autocorrelation's displacement
            
            % the new parameters of the line
            initLine_linearFit.m=tan(alpha_mean+90*pi/180);
            % should go through the displaced center
            initLine_linearFit.h=yCenter-xCenter*tan(alpha_mean+90*pi/180)-displacement(1)*tan(alpha_mean+90*pi/180)+displacement(2);
            
        case 'Quit'
            dlgBool=false;
            error('Quitting was requested.');
    end
end
%close();

%% Export
% exporting the rectangular region parameters for the next line investigation
lineDiam_lastParams.alpha_mean=alpha_mean+90*pi/180;
lineDiam_lastParams.displacement=displacement;
lineDiam_lastParams.xrange=xrange;
lineDiam_lastParams.yrange=yrange;
lineDiam_lastParams.sig=sig;
lineDiam_lastParams.sampLineDistThreshold=sampLineDistThreshold;
lineDiam_lastParams.nBins=nBins;
lineDiam_lastParams.polyDegree=polyDegree;
lineDiam_lastParams.surfFittingPower=surfFittingPower;
lineDiam_lastParams.x1=x1;
lineDiam_lastParams.y1=y1;
lineDiam_lastParams.linFunc_x=linFunc_x;
lineDiam_lastParams.linFunc_y=linFunc_y;
end


function [choice, skipPolinomialFittingBool] = choosedialog(stringCell, title, btn1_string, btn2_string, btn3_string, btn4_string)
% Dialog that lets th user set the initial line and set the region to be
% cut out.

%  source:
% https://www.mathworks.com/help/matlab/ref/dialog.html

% some of the window geometry's definition:
windowWidth=550;
windowHeight=300;
buttonMargin=30;
buttonWidth=200;
buttonHeight=25;


d = dialog('Position',[300 300 windowWidth windowHeight],'Name',title);

txt = uicontrol('Parent',d,...
    'Style','text',...
    'Position',[20 20+80+25 windowWidth-20 windowHeight-2*20-80-25],...
    'HorizontalAlignment', 'Left',...
    'String',stringCell);



btn1 = uicontrol('Parent',d,...
    'Style', 'pushbutton',...
    'Position',[buttonMargin 20 buttonWidth buttonHeight],...
    'String',btn1_string,...
    'Callback',@btn1_callback);

btn2 = uicontrol('Parent',d,...
    'Style', 'pushbutton',...
    'Position',[windowWidth-buttonWidth-buttonMargin 20 buttonWidth buttonHeight],...
    'String',btn2_string,...
    'Callback',@btn2_callback);

checkbox = uicontrol('Parent',d,...
    'Style','checkbox',...
    'Position',[windowWidth-buttonWidth-buttonMargin,55,buttonWidth,15],...
    'String','skip polinomial fitting');

btn3 = uicontrol('Parent',d,...
    'Style', 'pushbutton',...
    'Position',[buttonMargin 80 buttonWidth buttonHeight],...
    'String',btn3_string,...
    'Callback',@btn3_callback);

btn4 = uicontrol('Parent',d,...
    'Style', 'pushbutton',...
    'Position',[windowWidth-buttonWidth-buttonMargin 80 buttonWidth buttonHeight],...
    'String',btn4_string,...
    'Callback',@btn4_callback);


% Wait for d to close before running to completion
uiwait(d);

    function btn1_callback(source,event)
        %choice = char(popup_items(idx,:));
        choice = get(btn1, 'String');
        skipPolinomialFittingBool=logical(get(checkbox, 'Value'));
        delete(gcf)
    end

    function btn2_callback(source,event)
        choice = get(btn2, 'String');
        skipPolinomialFittingBool=logical(get(checkbox, 'Value'));
        delete(gcf)
    end

    function btn3_callback(source,event)
        choice = get(btn3, 'String');
        skipPolinomialFittingBool=logical(get(checkbox, 'Value'));
        delete(gcf)
    end

    function btn4_callback(source,event)
        choice = get(btn4, 'String');
        skipPolinomialFittingBool=logical(get(checkbox, 'Value'));
        delete(gcf)
    end

end