function [lineLength,lineHistogram,fig] = estimateLength(filename,coordinate,limitOption,Bins,lev50)
%ESTIMATELENGTH Summary of this function goes here
%   Detailed explanation goes here
nbins = Bins.bins;

fig = figure('Name',filename,'Visible','off');
switch limitOption
    case 'Gaussian'
        lineHistfit = histfit(coordinate, nbins, 'normal');
        hold on
        title(['Histfit using :', num2str(nbins), ' bins & normal distr. ', num2str(lev50)]);
        ylabel('Number of Locs in bins');
        [lineLength, tt, tl] = getEdges(lineHistfit(2).XData,lineHistfit(2).YData,lev50);
        % lineHistogram(2).XData; % X-axis data for the distribution curve
        % lineHistogram(2).YData; % Y-axis data for the distribution curve
        xline(tl);
        xline(tt);
        yline(max(lineHistfit(2).YData)*lev50);
        xlabel(['Diameter: ', num2str(lineLength), ' nm']);
        hold off
        % Store Histogram and Fitted Function
        lineHistogram.Histogram = lineHistfit(1);
        lineHistogram.FittedFunction = lineHistfit(2);
        % https://www.mathworks.com/help/stats/histfit.html
    case 'Kern'
        lineHistfit = histfit(coordinate, nbins, 'kernel');
        hold on
        title(['Histfit using :', num2str(nbins), ' bins & kern distr. ', num2str(lev50)]);
        ylabel('Number of Locs in bins');
        [lineLength, tt, tl] = getEdges(lineHistfit(2).XData,lineHistfit(2).YData,lev50);
        xline(tl);
        xline(tt);
        yline(max(lineHistfit(2).YData)*lev50);
        xlabel(['Diameter: ', num2str(lineLength), ' nm']);
        hold off
        % Store Histogram and Fitted Function
        lineHistogram.Histogram = lineHistfit(1);
        lineHistogram.FittedFunction = lineHistfit(2);
        % https://www.mathworks.com/help/stats/histfit.html
    case 'fwhm'
        % The distribution curve is displayed, but not used.
        lineHistfit = histfit(coordinate, nbins, 'kernel');
        hold on
        title(['Histogram using :', num2str(nbins), ' bins & fwhm ', num2str(lev50)]);
        ylabel('Number of Locs in bins');
        [lineLength, tt, tl] = fwhm(lineHistfit(1).XData,lineHistfit(1).YData,lev50);
        xline(tl);
        xline(tt);
        yline(max(lineHistfit(1).YData)*lev50);
        xlabel(['Diameter: ', num2str(lineLength), ' nm']);
        hold off
        % Store Histogram and Fitted Function
        lineHistogram.Histogram = lineHistfit(1);
        lineHistogram.FittedFunction = lineHistfit(2);
    case 'Rectangle'
        lineHistogram.Histogram = histogram(coordinate, Bins.edges);
        [lineHistogram.FittedFunction, fig]=lineDiameter_singlerectfit(...
            25, lineHistogram.Histogram, 'Gaussian', 'Rectangular', 20 );
        lineLength = lineHistogram.FittedFunction; % densFuncSize_fitted;
    case 'Circle'
        lineHistogram.Histogram = histogram(coordinate, Bins.edges);
        [lineHistogram.FittedFunction, fig]=lineDiameter_singlerectfit(...
            25, lineHistogram.Histogram, 'Gaussian', 'Circle', 20 );
        lineLength = lineHistogram.FittedFunction; % densFuncSize_fitted;
    case 'Ring'
        lineHistogram.Histogram = histogram(coordinate, Bins.edges);
        [lineHistogram.FittedFunction, fig]=lineDiameter_singlerectfit(...
            25, lineHistogram.Histogram, 'Gaussian', 'Ring', 20 );
        lineLength = lineHistogram.FittedFunction; % densFuncSize_fitted;
    case 'Disk'
        lineHistogram.Histogram = histogram(coordinate, Bins.edges);
        [lineHistogram.FittedFunction, fig]=lineDiameter_singlerectfit(...
            25, lineHistogram.Histogram, 'Gaussian', 'Disk', 20 );
        lineLength = lineHistogram.FittedFunction; % densFuncSize_fitted;
    case 'SignProc'
        % Using the Signal Processing Toolbox!
        % https://www.mathworks.com/help/signal/examples/measurement-of-pulse-and-transition-characteristics.html
        % https://www.mathworks.com/help/signal/ref/pulsewidth.html
    otherwise
        % https://terpconnect.umd.edu/~toh/spectrum/PeakFindingandMeasurement.htm
        disp('No option selected');
end


end

function [length, tt, tl] = getEdges(x, y, lev)
ymax = max(y);
% accept = (y >= ymax*lev);
tl = x(find(y >= ymax*lev, 1, 'first'));
tt = x(find(y >= ymax*lev, 1, 'last'));
length = tt - tl;
end