function [ densFuncWidth_fitted, fittedHistFigure ] = lineDiameter_singlerectfit( locPrecision, histHandler, linkerType, modelFuncType, linkerRad )

% modification of the 'lineDiameter_fit'

iterGaussianConvBool=false;
sampleType='single line';

%% Preparing the measured data of the line

histCounts = transpose(histHandler.BinCounts);
histEdges = transpose(histHandler.BinEdges);
    
histogramStruct.histCounts=histCounts;
histogramStruct.histEdges=histEdges;

%% The model function sampling settings

% refinement for the model function sampling:
binRefinement=10;
% Every bin (of the measured histogram data) should be refined to this many
% points. This determines only the sampling pont number, the sampling
% points will be equidistants, no matter the original bin sizes (thats why
% they should be equally sized).

[histBinN, binningRegion, sampling_N] = modelFunctionSampling.settings(histEdges ,binRefinement);

samplingSettings.binningRegion=binningRegion;
samplingSettings.sampling_N=sampling_N;
samplingSettings.binRefinement=binRefinement;
samplingSettings.convRad_N=[];
samplingSettings.histBinN=histBinN;

%% The model function settings
% Only those model function paramerters has to be set that the fitting is
% not performed on.
% The "modelFunctionSettings" should be in line with the
% "fittingParameterList" variable. What is defined in the
% "modelFunctionSettings" that is not respected in the
% "modelFunctionSettings" and will be overwrited (the fitting will be
% performed on that and will not be held constant). Every parameter
% neccessary parameter that is not defined in "fittingParameterList" (on
% which the fitting is not performed), should be defined in
% "modelFunctionSettings".

% initialize the structure for the model function parameters:
modelFunctionParameters=struct();

% the additional Gaussian smearing:
modelFunctionParameters.GaussianConvSigma=0;

% ring thickness in case of the "ring" structure
modelFunctionParameters.ringThickness=250;

% for predefined background:
%modelFunctionParameters.background=0;

% for predefined function area:
%modelFunctionParameters.area=sum(histCounts);
% if it is set, the fitting should not be performed on the "height"

% for rimless gap structure:
%modelFunctionParameters.rimWidth=0;
%modelFunctionParameters.rimHeight=0;

%% The convolution (if can be performed prior the fitting)

if ~iterGaussianConvBool
    % if the iteration should also run on the width of the Gaussian convolution

    % calculate the function with that the selected binding site density
    % function (model function) shall be convolved in order to get model
    % distribution to be fit on the data, accounts for the effect of the
    % linker radius and for the localisation precision
    modelFunc_sampling_dX=modelFunctionSampling.dX(binningRegion, sampling_N);
    [convFunc, convRad_N]=calculateConvolutionFunction(modelFunc_sampling_dX, linkerRad, locPrecision, linkerType, modelFunctionParameters.GaussianConvSigma);
    
    % TESTING:
    % convFunc=ones(31,1);
    % convRad_N=15;
else
    % calculate the convolution function inside the function called by the
    % iteration
    convFunc=[];
    convRad_N=[];
end


convolutionSettings.convFunc=convFunc;
convolutionSettings.locPrecision=locPrecision;
convolutionSettings.iterGaussianConvBool=iterGaussianConvBool;
convolutionSettings.linkerType=linkerType;
convolutionSettings.linkerRad=linkerRad;

%% Settings for the model function
samplingSettings.convRad_N=convRad_N;

modelFunctionStruct.sampleType=sampleType;
modelFunctionStruct.modelFuncType=modelFuncType;
modelFunctionStruct.modelFunctionSettings=modelFunctionParameters;
modelFunctionStruct.samplingSettings=samplingSettings;
modelFunctionStruct.convolutionSettings=convolutionSettings;

%% Fitting the model function

algorithm='fminsearch';
%algorithm='fmincon';
%algorithm='fminunc';

fittingParameterList={'width', 'height', 'position', 'background'};

if ~iterGaussianConvBool
    % do nothing
else
    fittingParameterList=[fittingParameterList, 'GaussianConvSigma'];
end

[initialParameters, constrains]=getInitialParameters.singleLine(histogramStruct, modelFuncType, modelFunctionParameters, fittingParameterList);

% fitting the model function to the histogram data of the measured
% distances
options = optimset('MaxIter',1000, 'TolFun', eps, 'TolX', eps);
%options = optimset();

[fittedParameters, FVAL]= lineDiameterFitting(histogramStruct, modelFunctionStruct, algorithm, fittingParameterList, initialParameters, constrains, options);

% the fitted parameters of the selected dye density function
densFuncWidth_fitted=fittedParameters.width;        % parameter determining its width, the FWHM
densFuncHeight_fitted=fittedParameters.height;         % parameter determining its peak value
densFuncBG_fitted=fittedParameters.background;          % parameter for constant background

if iterGaussianConvBool
    GaussianConvSigma_fitted=fittedParameters.GaussianConvSigma;
else
    GaussianConvSigma_fitted=modelFunctionParameters.GaussianConvSigma;
end

% update the model function parameters with the fitted ones
fittedParameterNames=fieldnames(fittedParameters);
for idxFittedParam=1:numel(fittedParameterNames)
    modelFunctionParameters.(fittedParameterNames{idxFittedParam})=fittedParameters.(fittedParameterNames{idxFittedParam});
end
modelFunctionStruct.modelFunctionSettings=modelFunctionParameters;


%% Visualizing the result

% update convolution function
if ~iterGaussianConvBool
    % do nothing
else
    modelFunc_sampling_dX=modelFunctionSampling.dX(binningRegion, sampling_N);
    [convFunc, convRad_N]=calculateConvolutionFunction(modelFunc_sampling_dX, linkerRad, locPrecision, linkerType, GaussianConvSigma_fitted);
    
    % update the convolution settigns structure
    convolutionSettings.convFunc=convFunc;
    convolutionSettings.convRad_N=convRad_N;
    modelFunctionStruct.convolutionSettings=convolutionSettings;
end


fittedHistFigure = lineDiameter_fit_visualization.singleLine(histogramStruct, modelFunctionStruct, densFuncWidth_fitted, densFuncBG_fitted, FVAL);


% show figure
figure(fittedHistFigure)

    
end