% script for launching the line diameter determination tools without GUI

if exist('OCTAVE_VERSION', 'builtin') ~= 0
    pkg load optim
    pkg load image
end

% adding the directories containing the required functions
run('addSubDirsToPath.m')

%% parameters for the line diameter determination
% parameters for automatic/manual evaluation
automaticSaveBool=false;
automaticProcessBool=false;
reusePreviousBool=false;

% clearing params
clearingParams.FrameMin=0;      % frame number bound
clearingParams.FrameMax=Inf;    % frame number bound
clearingParams.PSFmax=1.5;      % maximal accepted fitted Gaussian STDs of localizations in pixel size
clearingParams.PrecisionMax=25; % maximal accepted fitted Gaussian STDs of localizations in pixel size

% parameters that appear in the pop-up window for setting cut-out region
initParams.xrange=800;          % "x" size of the ROI's rectangle
initParams.yrange=400;          % "y" size of the ROI's rectangle
initParams.sig=20;              % STD of the Gaussian smearing
initParams.sampLineDistThreshold=400;   % maximum distance of lovcalizations considered from the central curve in nm
initParams.nBins=50;            % number of bins on the shown histogram
initParams.polyDegree=2;        % degree of the fitted central curve
initParams.surfFittingPower=1;  % raise to this power the localization density map

% structureType='Single line';
structureType='Double line';
% structureType='Double line (Fallback)';
% structureType='Gap';

% parameters of determining what kind of fitting should be performed (single line, double line, smeared double line or gap)
switch structureType
    case 'Single line'
        fitToMinBool=false;             % true for gap or unsmeared doule line
        doubleLinesBool=false;          % true for double lines
        linFitMethod='highestDensity';  % for single line or smeared double line, parameter only for the initial straight line fitting
    case 'Double line'
        % unsmeared double line
        fitToMinBool=true;              % true for gap or unsmeared doule line
        doubleLinesBool=true;           % true for double lines
        linFitMethod='surfaceMin';      % for unsmeared double line or gap, parameter only for the initial straight line fitting
    case 'Double line (Fallback)'
        % smeared double line
        fitToMinBool=false;             % true for gap or unsmeared doule line
        doubleLinesBool=true;           % true for double lines
        linFitMethod='highestDensity';  % for single line or smeared double line, parameter only for the initial straight line fitting
    case 'Gap'
        fitToMinBool=true;              % true for gap or unsmeared doule line
        doubleLinesBool=false;          % true for double lines
        linFitMethod='surfaceMin';      % for unsmeared double line or gap, parameter only for the initial straight line fitting
end

%% reading the section data files
sectionFullFileNames=[];
% pop-up window for opening one or several files
[sectionDataFileNames, sectionDataFilePath]= uigetfile({'*.mat'; '*.csv'}, 'Open one or several section data files.','MultiSelect','on');
% creating a cell list from the selected files
if sectionDataFilePath==0
    disp('User cancelled selection');
else
    % number of the chosen histogram data files
    if iscell(sectionDataFileNames)
        N_histDataFiles=numel(sectionDataFileNames);   % if multiple files were chosen, the names are stored in a cell array
    else
        N_histDataFiles=1;      % if a single file was chosen, the name is stored in a string
    end
    sectionFullFileNames=cell(1,N_histDataFiles);
    if N_histDataFiles==1
        %disp(['Regex selected ', fullfile(sectionsPath,sectionDataFiles(idxSection).name)]);
        [~,sectionName,~]=fileparts(sectionDataFileNames);
        sectionFullFileNames{1}=fullfile(sectionDataFilePath, sectionDataFileNames);
    elseif N_histDataFiles>1
        for idxSection = 1:N_histDataFiles
            %disp(['Regex selected ', fullfile(sectionsPath,sectionDataFiles(idxSection).name)]);
            [~,sectionName,~]=fileparts(sectionDataFileNames{idxSection});
            sectionFullFileNames{idxSection}=fullfile(sectionDataFilePath, sectionDataFileNames{idxSection});
        end
    end
end

% To read all files in a folder
% folder = 'D:\Sls700_B2\';
% listOfFiles = dir([folder,'*','.mat']);
% for idxList=1:length(listOfFiles)
%     currectListName=[folder,listOfFiles(idxList).name];
%     sectionFullFileNames{idxList}=currectListName;
% end

%% calling the line diameter determinator tools
rainSTORM_extras_lineDiameter(sectionFullFileNames, initParams, clearingParams, fitToMinBool, doubleLinesBool, linFitMethod, automaticSaveBool, automaticProcessBool, reusePreviousBool);
disp('Done');
