function [mergeDataPath, SupResIm] = rainSTORM_extras_lineDiameter_merge(mergeLineDiamHistData,mergeParams)
% This function is used to align the individual single band (single line),
% double band (double line) or gap structures and merge them into a single
% image.
% If 'onlyfinal' is false, create the centered images for all results

%% Loop through the sections
mergeData = cell(length(mergeLineDiamHistData),4);
summedxy = [];
    for idxSection = 1:length(mergeLineDiamHistData)
        currentHistFile=mergeLineDiamHistData{idxSection};
        [sectionMergePath,sectionMergeName,currentHistFileExt]=fileparts(currentHistFile);

        switch currentHistFileExt
            case '.mat'
                mergeHist=load(currentHistFile,'-mat','lineDiam_histData');
            otherwise
                error('Invalid file type.');
        end
        % Load complete

        %% Calculate bounds and poly line1
        [minX, maxX] = bounds(mergeHist.lineDiam_histData.lineDiam_lastParams{1,1}.LOC_X_ALPHA);
        XLine = minX-mergeParams.bound:mergeParams.stepsize:maxX+mergeParams.bound;
        YLine = polyval(mergeHist.lineDiam_histData.lineDiam_lastParams{1,1}.fit_coeffs, XLine);
        % End of line1 calculation

        %% Calculate distance from line1 for each point
        curvexy = [XLine' YLine'];
        mapxy = [mergeHist.lineDiam_histData.lineDiam_lastParams{1,1}.LOC_X_ALPHA...
            mergeHist.lineDiam_histData.lineDiam_lastParams{1,1}.LOC_Y_ALPHA];
        
        [splinexy,d] = distance2curve(curvexy,mapxy,mergeParams.interp);
        
        side = splinexy(:, 2) >=  mapxy(:,2);
        remappedxy = zeros(size(mapxy));
        remappedxy(:, 1) = mapxy(:,1);
        remappedxy(side, 2) = d(side);
        remappedxy(~side, 2) = -d(~side);
        mergeData(idxSection,1) = {sectionMergeName};
        mergeData(idxSection,2) = {remappedxy};
        summedxy = [ summedxy; remappedxy ];
    end
    
    if (mergeParams.crop)   % Calculate the crop
        accept = summedxy(:,1) >= -mergeParams.cropX/2 & summedxy(:,1) <= mergeParams.cropX/2 & ...
            summedxy(:,2) >= -mergeParams.cropY/2 & summedxy(:,2) <= mergeParams.cropY/2;            
    end

    % create the directory where the results will be saved
    mkdir(sectionMergePath, 'mergeData');
    
    % Shift to the center
    shiftedxy(:,1) = summedxy(:,1) + mergeParams.sizeX/2;
    shiftedxy(:,2) = summedxy(:,2) + mergeParams.sizeY/2;
    
    % Do the crop
    if (mergeParams.crop)   % Do the crop
        shiftedxy = shiftedxy(accept,:);
    end
    
    % Create SupRes image (rainSTORM simple histogram reconstruction).
    nRows = ceil(mergeParams.sizeX/mergeParams.pixelSize);  % Size of super-res reconstruction
    nCols = ceil(mergeParams.sizeY/mergeParams.pixelSize);  % Ceil() allows awkward scalings
    SupResIm = uint16(zeros(nRows,nCols));                  % 2^16 is unlikely to overflow
    
    myPosits = shiftedxy/mergeParams.pixelSize;             % Scale localisations to super-res grid
    myPosits = ceil(myPosits);                              % Digitise localisations to super-res grid

    for lpSRI = 1:size(myPosits,1)
        if ( myPosits(lpSRI,1)<1 || myPosits(lpSRI,2)<1 || ...
               myPosits(lpSRI,1)>nRows || myPosits(lpSRI,2)>nCols )
            continue; % Skip over positions if they are outside reconstruction (JH)
        end
    
        SupResIm( myPosits(lpSRI,1)  , myPosits(lpSRI,2)  ) = ...
        SupResIm( myPosits(lpSRI,1)  , myPosits(lpSRI,2)  ) +1;
        % Elementwise binning. Not very elegant, I admit.
    end % End of allocating dyes to pixels of the reconstruction
    
    %%% Ugly, but do the same again for each image.
    for idxMergeData = 1:size(mergeData,1)
        if (mergeParams.crop)   % Calculate the crop
            accept_single = mergeData{idxMergeData,2}(:,1) >= -mergeParams.cropX/2 & mergeData{idxMergeData,2}(:,1) <= mergeParams.cropX/2 & ...
                mergeData{idxMergeData,2}(:,2) >= -mergeParams.cropY/2 & mergeData{idxMergeData,2}(:,2) <= mergeParams.cropY/2;
        end
        mergeData{idxMergeData,3}(:,1) = mergeData{idxMergeData,2}(:,1) + mergeParams.sizeX/2;
        mergeData{idxMergeData,3}(:,2) = mergeData{idxMergeData,2}(:,2) + mergeParams.sizeY/2;
        singleSupResIm = uint16(zeros(nRows,nCols));
        if (mergeParams.crop)   % Do the crop
            mergeData{idxMergeData,4}(:,1) = mergeData{idxMergeData,3}(accept_single,1);
            mergeData{idxMergeData,4}(:,2) = mergeData{idxMergeData,3}(accept_single,2);
            singlemyPosits = mergeData{idxMergeData,4}/mergeParams.pixelSize;
        else
            singlemyPosits = mergeData{idxMergeData,3}/mergeParams.pixelSize;
        end
        singlemyPosits = ceil(singlemyPosits);
        for lpSRI = 1:size(singlemyPosits,1)
            if ( singlemyPosits(lpSRI,1)<1 || singlemyPosits(lpSRI,2)<1 || ...
                singlemyPosits(lpSRI,1)>nRows || singlemyPosits(lpSRI,2)>nCols )
                continue; % Skip over positions if they are outside reconstruction (JH)
            end

        singleSupResIm( singlemyPosits(lpSRI,1)  , singlemyPosits(lpSRI,2)  ) = ...
        singleSupResIm( singlemyPosits(lpSRI,1)  , singlemyPosits(lpSRI,2)  ) +1;
        % Elementwise binning. Not very elegant, I admit.
        end % End of allocating dyes to pixels of the reconstruction
        if ~(mergeParams.onlyfinal)
            imwrite(uint8(singleSupResIm),[fullfile(sectionMergePath,'mergeData',mergeData{idxMergeData,1}),'-MERGEImage.png']);
        end
        if (mergeParams.outputInFigure)
            ifmAnalyser_save_figure(mergeData{idxMergeData,3}(:,1),mergeData{idxMergeData,3}(:,2),...
                [fullfile(sectionMergePath,'mergeData',mergeData{idxMergeData,1}),'-MERGEFigure.fig']);
        end
        % Do the crop, but don't create superresolved image for the crop
        mergeData{idxMergeData,5} = singleSupResIm;
    end
    
    % Create exports and returns
    mergeDataPath = fullfile(sectionMergePath,'mergeData','mergeOutput.mat');
    
    mergeOutput.SupResIm = SupResIm;
    mergeOutput.mergeDataPath = mergeDataPath;
    mergeOutput.mergeData = mergeData;
    mergeOutput.summedxy = summedxy;
    mergeOutput.locs = length(summedxy);
    mergeOutput.pixelSize = mergeParams.pixelSize;
    mergeOutput.sizeX = mergeParams.sizeX;
    mergeOutput.sizeY = mergeParams.sizeY;
    
    % saving the data

    save(mergeDataPath,'mergeOutput', '-mat');
    save(fullfile(sectionMergePath,'mergeData','concatenated_xydata.mat'),'mergeData', '-mat');
    imwrite(uint16(SupResIm),[fullfile(sectionMergePath,'mergeData','output'),'-MERGEImage.png']);
    if (mergeParams.outputInFigure)
        ifmAnalyser_save_figure(summedxy(:,1),summedxy(:,2),...
            [fullfile(sectionMergePath,'mergeData','output'),'-MERGEFigure.fig']);
    end
    
end

